@@ -1,15 +1,14 @@
 package PICA::Schema;
 use v5.14.1;
 
-our $VERSION = '1.15';
+our $VERSION = '1.16';
 
 use Exporter 'import';
-our @EXPORT_OK = qw(field_identifier check_value);
+our @EXPORT_OK = qw(field_identifier check_value clean_pica);
 
 use Scalar::Util qw(reftype);
 use Storable qw(dclone);
 use PICA::Error;
-use PICA::Data;
 
 sub new {
     my ($class, $schema) = @_;
@@ -21,26 +20,28 @@ sub check {
 
     my @errors;
     $record
-        = PICA::Data::clean_pica($record, error => sub {push @errors, shift});
-    return @errors unless $record;
+        = clean_pica($record, error => sub {push @errors, shift}, %options)
+        or return @errors;
 
     $options{counter} = {};
 
     my %field_identifiers;
     for my $field (@$record) {
-        $field_identifiers{field_identifier($field)} = 1;
-        my $error = $self->check_field($field, %options);
+        my $id = $self->field_identifier($field);
+        $field_identifiers{$id} = 1;
+        my $error = $self->check_field($field, %options, _field_id => $id);
         if ($error and !grep {$_ eq $error} @errors) {
             push @errors, $error;
         }
     }
 
+    # check whether required fields exist
     for my $id (keys %{$self->{fields}}) {
         my $field = $self->{fields}{$id};
         if ($field->{required} && !$field_identifiers{$id}) {
             push @errors,
                 PICA::Error->new(
-                [substr($id, 0, 4), length $id gt 4 ? substr($id, 5) : undef],
+                [substr($id, 0, 4), length $id > 4 ? substr($id, 5) : undef],
                 required => 1
                 );
         }
@@ -52,7 +53,7 @@ sub check {
 sub check_field {
     my ($self, $field, %opts) = @_;
 
-    my $id   = field_identifier($field);
+    my $id = $opts{_field_id} || $self->field_identifier($field);
     my $spec = $self->{fields}{$id};
 
     if ($opts{allow_deprecated}) {
@@ -187,8 +188,22 @@ sub check_value {
 }
 
 sub field_identifier {
+    my $fields = ref $_[0] eq 'PICA::Schema' ? shift->{fields} : undef;
     my ($tag, $occ) = @{$_[0]};
-    (($occ // '') ne '' and substr($tag, 0, 1) eq '0') ? "$tag/$occ" : $tag;
+
+    $occ
+        = (substr($tag, 0, 1) ne '2' && defined $occ && $occ ne '')
+        ? sprintf("%02d", $occ)
+        : '';
+
+    if ($fields && $occ ne '' && !exists $fields->{"$tag/$occ"}) {
+        for my $id (keys %$fields) {
+            return $id
+                if $id =~ /^$tag\/(..)-(..)$/ && $occ >= $1 && $occ <= $2;
+        }
+    }
+
+    return $occ ne '' ? "$tag/$occ" : $tag;
 }
 
 sub TO_JSON {
@@ -209,6 +224,73 @@ sub abbreviated {
     return $abbr;
 }
 
+sub clean_pica {
+    my ($record, %options) = @_;
+
+    my $ok      = 1;
+    my $handler = $options{error};
+    my $error   = exists $options{error}
+        ? sub {
+        if ($handler) {
+            $handler->(PICA::Error->new($_[1] || [], message => $_[0]));
+        }
+        $ok = 0;
+        }
+        : sub {say STDERR shift; $ok = 0};
+
+    $record = $record->{record} if reftype $record eq 'HASH';
+
+    if (reftype $record ne 'ARRAY') {
+        $error->('PICA record must be array reference');
+    }
+    elsif (!@$record && !$options{ignore_empty_records}) {
+        $error->('PICA record should not be empty');
+    }
+
+    return unless $ok;
+
+    my @filtered;
+
+    for my $field (@$record) {
+        if (reftype $field ne 'ARRAY') {
+            $error->('PICA field must be array reference');
+            return
+        }
+
+        my ($tag, $occ, @sf) = @$field;
+
+        if ($tag !~ /^[012.][0-9.][0-9.][A-Z@.]$/) {
+            $error->("Malformed PICA tag: $tag", $field);
+        }
+
+        if ($occ) {
+            if ($occ !~ /^[0-9]{1,3}$/) {
+                $error->("Malformed occurrence: $occ", $field);
+            }
+            elsif (substr($tag, 0, 1) ne '2' && length $occ eq 3) {
+                $error->(
+                    "Three digit occurrences only allowed on PICA level 2",
+                    $field
+                );
+            }
+        }
+
+        continue if $options{ignore_subfields};
+
+        while (@sf) {
+            my $code  = shift @sf;
+            my $value = shift @sf;
+
+            $error->("Malformed PICA subfield: $code", $field)
+                if $code !~ /^[_A-Za-z0-9]$/;
+            $error->("PICA subfield \$$code must be non-empty string", $field)
+                if $value !~ /^./;
+        }
+    }
+
+    return $record if $ok;
+}
+
 1;
 __END__
 
@@ -245,7 +327,8 @@ language|https://format.gbv.de/schema/avram/specification>, for instance:
 See L<PICA::Schema::Builder> to automatically construct schemas from PICA
 sample records.
 
-Schema information can be included in PICA XML with L<PICA::Writer::XML>.
+Schema information can also be used for documentation of records with
+L<PICA::Writer::Fields> and L<PICA::Writer::XML>.
 
 =head1 METHODS
 
@@ -301,14 +384,42 @@ C<check>. Returns a L<PICA::Error> on schema violation.
 
 =head2 abbreviated
 
-Return an abbreviated data structure of the schema without inferable fields such as C<tag>, C<occurrence> and C<code>.
+Return an abbreviated data structure of the schema without inferable fields
+such as C<tag>, C<occurrence> and C<code>.
 
 =head1 FUNCTIONS
 
-=head2 field_identifier( $field )
+=head2 clean_pica( $record[, %options] )
+
+Syntactically check a PICA record and return it as array of arrays on success.
+Syntactic check is performed on schema validation before checking the record
+against a schema and before writing a record.
+
+Options include:
+
+=over
+
+=item error
+
+Error handler, prints instances of L<PICA::Error> to STDERR by default. Use
+C<undef> to ignore all errors.
+
+=item ignore_empty_records
+
+Don't emit an error if the record has no fields.
+
+=item ignore_subfields
+
+Don't check subfields.
+
+=back
+
+=head2 field_identifier( [$schema, ] $field )
 
 Return the field identifier of a given PICA field. The identifier consists of
-field tag and optional occurrence if the tag starts with C<0>.
+field tag and optional occurrence. If this function is used as method of a
+schema, the field_identifier may contain an occurrence ranges instead of the
+plain occurrence.
 
 =head2 check_value( $value, $schedule [, %options ] )
 
